@@ -376,5 +376,161 @@ def parmdat(filename):
         
     return atoms, bonds, angles, propers, impropers, LJs
         
+sys.modules['__main__'].__dict__["loadparmdat"] = parmdat 
 
-sys.modules['__main__'].__dict__["loadparmdat"] = parmdat 
\ No newline at end of file
+##########################################################################
+#GROMACS Format
+##########################################################################
+
+class GROMACS_TOPOLOGY_ITERATOR():
+    def __init__(self, filename = None, macros = {}):
+        self.files = []
+        self.filenames = []
+        self.defined_macros = macros
+        if filename:
+            self.Add_Iterator_File(filename)
+    def Add_Iterator_File(self, filename):
+        if self.files:
+            filename = os.path.abspath(os.path.join(os.path.dirname(self.filenames[-1]), filename.replace('"', '')))
+        else:
+            filename = os.path.abspath(filename.replace('"', ''))
+            
+        f = open(filename)
+        self.files.append(f)
+        self.filenames.append(filename)
+    def __iter__(self):
+        self.flag = ""
+        self.macro_define_stat = []
+        return self
+    def __next__(self):
+        while self.files:
+            f = self.files[-1]
+            line = f.readline()
+            
+            if line:
+                line = line.strip()
+                comment = line.find(";")
+                if comment >= 0:
+                    line = line[:comment]
+                while line and line[-1] == "\\":
+                    line = line[:-1] + " " + next(self).strip()        
+                if not line:
+                    line = next(self)
+                if line[0] == "#":         
+                    words = line.split()
+                    if words[0] == "#ifdef":
+                        macro = words[1]
+                        if self.macro_define_stat and self.macro_define_stat[-1] == False:
+                            self.macro_define_stat.append(False)
+                        elif macro in self.defined_macros.keys():
+                            self.macro_define_stat.append(True)
+                        else:
+                            self.macro_define_stat.append(False)
+                    elif words[0] == "#else":
+                        if len(self.macro_define_stat) <= 1 or self.macro_define_stat[-2] != False:
+                            self.macro_define_stat[-1] = not self.macro_define_stat[-1]
+                    elif words[0] == "#endif":
+                        self.macro_define_stat.pop()    
+                    elif self.macro_define_stat and self.macro_define_stat[-1] == False:
+                        next(self)
+                    elif words[0] == "#define":
+                        if len(words) > 2:
+                            self.defined_macros[words[1]] = line[line.find(words[1]) + len(words[1]):].strip()
+                        else:    
+                            self.defined_macros[words[1]] = ""
+                    elif words[0] == "#include":
+                        self.Add_Iterator_File(words[1])
+                    elif words[0] == "#undef":
+                        self.defined_macros.pop(words[1])
+                    elif words[0] == "#error":
+                        raise Exception(line)
+                    line = next(self)
+                elif self.macro_define_stat and self.macro_define_stat[-1] == False:
+                    line = next(self)
+                elif "[" in line and "]" in line:
+                    self.flag = line[1:-1].strip()
+                    line = next(self)
+                for macro, tobecome in self.defined_macros.items():
+                    line = line.replace(macro, tobecome)
+                return line
+            else:
+                f.close()
+                self.files.pop()
+                self.filenames.pop()
+                continue
+        raise StopIteration
+            
+def ffitp(filename, macros = {}):
+    iterator = GROMACS_TOPOLOGY_ITERATOR(filename, macros)
+    output = {}
+    output["nb14"] = "name  kLJ  kee\n"
+    output["atomtypes"] = "name mass charge[e] LJtype\n"
+    output["bonds"] = "name b[nm] k[kJ/mol·nm^-2]\n"
+    output["angles"] = "name b[degree] k[kJ/mol·rad^-2]\n"
+    output["Urey-Bradley"] = "name b[degree] k[kJ/mol·rad^-2] r13[nm] kUB[kJ/mol·nm^-2]\n"
+    output["dihedrals"] = "name phi0[degree] k[kJ/mol] periodicity  reset\n"
+    output["impropers"] = "name phi0[degree] k[kJ/mol·rad^-2]\n"
+    output["cmaps"] = {}
+    for line in iterator:
+        if iterator.flag == "":
+            continue
+        elif iterator.flag == "defaults":
+            words = line.split()
+            assert int(words[0]) == 1, "SPONGE Only supports Lennard-Jones now"
+            if int(words[1]) == 1:
+                output["LJ"] = "name A[kJ/mol·nm^6] B[kJ/mol·nm^12]\n"
+                output["nb14_extra"] = "name A[kJ/mol·nm^6] B[kJ/mol·nm^12] kee\n"
+            else:
+                output["LJ"] = "name sigma[nm] epsilon[kJ/mol] \n"
+                output["nb14_extra"] = "name sigma[nm] epsilon[kJ/mol] kee\n"
+            fudgeLJ = float(words[3])
+            fudgeQQ = float(words[4])
+            if words[2] == "yes":
+                output["nb14"] += "X-X {fudgeLJ} {fudgeQQ}\n".format(fudgeLJ=fudgeLJ, fudgeQQ=fudgeQQ)
+                
+        elif iterator.flag == "atomtypes":
+            words = line.split()
+            output["atomtypes"] += "{type} {mass} {charge} {type}\n".format(type=words[0], mass = float(words[2]), charge = float(words[3]))
+            output["LJ"] += "{type}-{type} {V} {W}\n".format(type=words[0], V=float(words[5]), W=float(words[6]))
+        elif iterator.flag == "pairtypes":
+            words = line.split()
+            if len(words) <= 3:
+                output["nb14"] +=  "{atom1}-{atom2} {kLJ} {kee}\n".format(atom1 = words[0], atom2 = words[1], kLJ = fudgeLJ, kee = fudgeQQ)
+            elif words[2] == "1":
+                output["nb14_extra"] += "{atom1}-{atom2} {V} {W} {kee}\n".format(atom1 = words[0], atom2 = words[1], V = float(words[3]), W = float(words[4]), kee = fudgeQQ)
+                output["nb14"] += "{atom1}-{atom2} 0 0\n".format(atom1 = words[0], atom2 = words[1])
+            elif words[2] == "2":
+                raise NotImplementedError
+        elif iterator.flag == "bondtypes":
+            words = line.split()
+            func = words[2]
+            if func == "1":
+                output["bonds"] += "{atom1}-{atom2} {b} {k}\n".format(atom1 = words[0], atom2 = words[1], b = float(words[3]), k = float(words[4]))
+            else:
+                raise NotImplementedError
+        elif iterator.flag == "angletypes":
+            words = line.split()
+            func = words[3]
+            if func == "1":
+                output["angles"] += "-".join(words[:3]) + " " + " ".join(words[4:]) + "\n"
+            elif func == "5":
+                output["Urey-Bradley"] += "-".join(words[:3]) + " " + " ".join(words[4:])  + "\n"
+            else:
+                raise NotImplementedError
+        elif iterator.flag == "dihedraltypes":
+            words = line.split()
+            func = words[4]
+            if func == "1":
+                output["dihedrals"] += "-".join(words[:4]) + " " + " ".join(words[5:]) + " 0\n"
+            elif func == "2":
+                output["impropers"] += "-".join([words[1], words[2], words[0], words[3]]) + " " + " ".join(words[5:]) + "\n"
+            elif func == "9":
+                for i in range(5,len(words),20):
+                    output["dihedrals"] += "-".join(words[:4]) + " " + " ".join(words[i:i+3]) + " 0\n"
+        elif iterator.flag == "cmaptypes":
+            words = line.split()
+            output["cmaps"]["-".join(words[:5])] = {"resolution": int(words[7]), "parameters":list(map(float, words[8:]))}
+        elif iterator.flag == "nonbond_params":
+            words = line.split()
+            output["LJ"] += "{type1}-{type2} {V} {W}\n".format(type1=words[0], type2=words[1], V=float(words[3]), W=float(words[4]))
+    return output
\ No newline at end of file
